
library fields.field_item;

import 'dart:async';
import 'package:tags/builder.dart';
import 'package:tags/custom.dart';
import 'package:tags/html.dart';
import 'package:fields/field.dart';
import 'package:fields/field_editor.dart';
import 'package:fields/theme.dart';
import 'field_list_view.dart';

final Tag fieldItem = 
    ([Build b, List args]) => 
        build(() => bind(new FieldItem(args[0]), args[1]), b);    

class Resources {
  String get style =>
"""
.expander {
  background-color:transparent;
  border-style:none;
  color:${theme.colors.text};
  float:left;   
  width:16px;
  padding:0;
  margin-top:4px;
}
.editorbox {
  display:block; 
  overflow:hidden;  
}
""";
  const Resources();
}

class FieldItem implements HasElement {  
  
  Element _element;
  Element get element => _element;
  
  final FieldListView _list = new FieldListView();  
  ToggleButton _toggle;
  
  final Field _field;
  Field get field => _field;
           
  FieldEditor _editor;  
  bool get _expanded => _toggle.down;
  
  FieldItem(this._field, [Resources res = const Resources()]) {    
    _editor = new FieldEditor.forField(_field);
    
    _element =
        span(() {
          shadow(() {
            style()..text = res.style;            
            toggleButton(() {
              span()..innerHtml = '&#9655;'..classes = ['up'];
              span()..innerHtml = '&#9698;'..classes = ['down'];
            }, (ToggleButton toggle) {
              _toggle = toggle;
              _toggle.states.listen((_) => render());
            })..classes.add('expander');
            span(() {
              append(_editor.element);
            })..classes.add('editorbox');
            append(_list.element);
          });
        });
        
    field.values.listen((_) => _render());
    _render();
  }
  
  void render() {
    if (_expanded) _list.render(_field.children);
    else _list.render([]);
    _render();
  }
  
  void _render() {
    _toggle.element.hidden = _field.children.isEmpty;
    _list.element.hidden = !_expanded;    
  }  
}
