
library fields.field_list_view;

import 'package:tags/builder.dart';
import 'package:tags/custom.dart';
import 'package:tags/html.dart';
import 'package:fields/field.dart';
import 'package:fields/theme.dart';
import 'field_item.dart';

final Tag fieldListView = 
    ([Build b, Bind onBind]) => 
        build(() => bind(new FieldListView(), onBind), b);

class Resources {
  String get style =>
"""
table {
  border-collapse:collapse;
  border-top:1px solid ${theme.colors.border};
  border-left:1px solid ${theme.colors.border};
  margin:-1px;            
  padding:0px;
  table-layout:fixed;
  width:100%;
}
td {
  background-color: ${theme.colors.fieldBackground};
  border-top:1px solid ${theme.colors.border};
  border-left:1px solid ${theme.colors.border};
  color: ${theme.colors.text};
  vertical-align:text-top;
  margin:0px;
  overflow:hidden;
  padding:0px;
  text-overflow:ellipsis;
}
.namebox {
  padding-left:4px;
  width:20%;
}
""";
  const Resources();
}

class FieldListView implements HasElement {
  
  final Map<Field, List> _cache = new Map<Field, List>();
  Element _element;
  Element get element => _element;
    
  Element _tbody;
  
  FieldListView([Resources res = const Resources()]) {
    _element =
        span(() {
          style()
            ..scoped = true
            ..text = res.style;
          table(() => _tbody = tbody());
        });
  }

  void clearCache() => _cache.clear();
  
  void render(List<Field> fields) {
    _tbody.children.clear();
    for (var f in fields) {        
      if (_cache.containsKey(f)) {
        _tbody.append(_cache[f][0]);
        _cache[f][1].render();
      } else {
        FieldItem item;
        final el = tr(() {
          td(() => span()..text = f.displayName)
            ..title = f.displayName
            ..classes = ['namebox'];
          td(() => fieldItem(null, [f, (FieldItem model) => item = model]));
        });
        _cache[f] = [el, item];
        _tbody.append(el);
      }
    }
  }
}
