
library fields.mirrors;

import 'dart:mirrors';

// TODO(rms): remove this once the VM supports generic type parameter literals.
// https://code.google.com/p/dart/issues/detail?id=11345
bool get _isDart2Js => (1 is double);

// TODO(rms): https://code.google.com/p/dart/issues/detail?id=6433
Type typeOf(TypeMirror tm) {
  if (tm.simpleName == const Symbol('bool'))          return bool;
  else if (tm.simpleName == const Symbol('DateTime')) return DateTime;
  else if (tm.simpleName == const Symbol('double'))   return double;
  else if (tm.simpleName == const Symbol('Duration')) return Duration;  
  else if (tm.simpleName == const Symbol('dynamic') && _isDart2Js)
    return Dynamic;
  else if (tm.simpleName == const Symbol('int'))      return int;
  else if (tm.simpleName == const Symbol('List'))     return List;
  else if (tm.simpleName == const Symbol('Map'))      return Map;
  else if (tm.simpleName == const Symbol('num'))      return num;
  else if (tm.simpleName == const Symbol('String'))   return String;
  else                                                return null;
}

class _MakerOf<T> {
  const _MakerOf();
  get type => T;
}

final Type Dynamic = const _MakerOf().type;

bool isSimpleType(Type t) {
  switch(t) {
    case bool:
    case double:
    case int:    
    case List:
    case Map:
    case Object:
    case num:
    case String:
      return true;
    default:
      return _isDart2Js && Dynamic == t;
  }
}

final _objectClassMirror = reflectClass(Object);

/// Returns `true` of the given [type] and the given [other] represent the same
/// type, or if the [type] is in the inheritance hierarchy of [other], or if
/// [other] implements the interface of [type]; else returns `false`.
bool isAssignableFrom(ClassMirror type, ClassMirror other) {
  if (type == null) throw new ArgumentError('type must not be null.');
  if (other == null) throw new ArgumentError('other must not be null.');
  if (_classMirrorEquals(other, type)) return true;
  if (_classMirrorEquals(other.superclass, type)) return true;
  if (other.superinterfaces.any((i) => _classMirrorEquals(i, type)))
    return true;
  // TODO(rms): https://code.google.com/p/dart/issues/detail?id=5794 
  if (_classMirrorEquals(other.superclass, _objectClassMirror)) {
    return _classMirrorEquals(type, _objectClassMirror);
  }
  return isAssignableFrom(type, other.superclass);
}

// TODO(rms): https://code.google.com/p/dart/issues/detail?id=5505
bool _classMirrorEquals(ClassMirror a, ClassMirror b) {
  assert(a != null);
  assert(b != null);
  return a.qualifiedName == b.qualifiedName;
}
